@@ -13,17 +13,14 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-from __future__ import absolute_import, unicode_literals
+from __future__ import division, absolute_import, print_function
 
 import os
 
-import beets.library
 from beets.util import confit
 
-__version__ = '1.3.17'
-__author__ = 'Adrian Sampson <adrian@radbox.org>'
-
-Library = beets.library.Library
+__version__ = u'1.4.1'
+__author__ = u'Adrian Sampson <adrian@radbox.org>'
 
 
 class IncludeLazyConfig(confit.LazyConfig):
