@@ -19,18 +19,19 @@ music player.
 
 from __future__ import division, absolute_import, print_function
 
+import six
 import sys
 import time
-import thread
+from six.moves import _thread
 import os
 import copy
-import urllib
+from six.moves import urllib
 from beets import ui
 
 import gi
-from gi.repository import GLib, Gst
-
 gi.require_version('Gst', '1.0')
+from gi.repository import GLib, Gst  # noqa: E402
+
 
 Gst.init(None)
 
@@ -128,9 +129,9 @@ class GstPlayer(object):
         path.
         """
         self.player.set_state(Gst.State.NULL)
-        if isinstance(path, unicode):
-            path = path.encode('utf8')
-        uri = 'file://' + urllib.quote(path)
+        if isinstance(path, six.text_type):
+            path = path.encode('utf-8')
+        uri = 'file://' + urllib.parse.quote(path)
         self.player.set_property("uri", uri)
         self.player.set_state(Gst.State.PLAYING)
         self.playing = True
@@ -164,7 +165,7 @@ class GstPlayer(object):
             loop = GLib.MainLoop()
             loop.run()
 
-        thread.start_new_thread(start, ())
+        _thread.start_new_thread(start, ())
 
     def time(self):
         """Returns a tuple containing (position, length) where both
