@@ -90,6 +90,9 @@ class PlayPlugin(BeetsPlugin):
                 command_str = command_str.replace(ARGS_MARKER, opts.args)
             else:
                 command_str = u"{} {}".format(command_str, opts.args)
+        else:
+            # Don't include the marker in the command.
+            command_str = command_str.replace(" " + ARGS_MARKER, "")
 
         # Perform search by album and add folders rather than tracks to
         # playlist.
@@ -128,7 +131,7 @@ class PlayPlugin(BeetsPlugin):
                 u'You are about to queue {0} {1}.'.format(
                     len(selection), item_type)))
 
-            if ui.input_options(('Continue', 'Abort')) == 'a':
+            if ui.input_options((u'Continue', u'Abort')) == 'a':
                 return
 
         ui.print_(u'Playing {0} {1}.'.format(len(selection), item_type))
@@ -137,8 +140,7 @@ class PlayPlugin(BeetsPlugin):
         else:
             open_args = [self._create_tmp_playlist(paths)]
 
-        self._log.debug(u'executing command: {} {}', command_str,
-                        b' '.join(open_args))
+        self._log.debug(u'executing command: {} {!r}', command_str, open_args)
         try:
             util.interactive_open(open_args, command_str)
         except OSError as exc:
