@@ -14,8 +14,7 @@
 # included in all copies or substantial portions of the Software.
 
 """A Web interface to beets."""
-from __future__ import (division, absolute_import, print_function,
-                        unicode_literals)
+from __future__ import division, absolute_import, print_function
 
 from beets.plugins import BeetsPlugin
 from beets import ui
@@ -56,11 +55,13 @@ def _rep(obj, expand=False):
         return out
 
 
-def json_generator(items, root):
+def json_generator(items, root, expand=False):
     """Generator that dumps list of beets Items or Albums as JSON
 
     :param root:  root key for JSON
     :param items: list of :class:`Item` or :class:`Album` to dump
+    :param expand: If true every :class:`Album` contains its items in the json
+                   representation
     :returns:     generator that yields strings
     """
     yield '{"%s":[' % root
@@ -70,10 +71,16 @@ def json_generator(items, root):
             first = False
         else:
             yield ','
-        yield json.dumps(_rep(item))
+        yield json.dumps(_rep(item, expand=expand))
     yield ']}'
 
 
+def is_expand():
+    """Returns whether the current request is for an expanded response."""
+
+    return flask.request.args.get('expand') is not None
+
+
 def resource(name):
     """Decorates a function to handle RESTful HTTP requests for a resource.
     """
@@ -83,7 +90,7 @@ def resource(name):
             entities = [entity for entity in entities if entity]
 
             if len(entities) == 1:
-                return flask.jsonify(_rep(entities[0]))
+                return flask.jsonify(_rep(entities[0], expand=is_expand()))
             elif entities:
                 return app.response_class(
                     json_generator(entities, root=name),
@@ -91,7 +98,7 @@ def resource(name):
                 )
             else:
                 return flask.abort(404)
-        responder.__name__ = b'get_%s' % name.encode('utf8')
+        responder.__name__ = 'get_{0}'.format(name)
         return responder
     return make_responder
 
@@ -102,10 +109,13 @@ def resource_query(name):
     def make_responder(query_func):
         def responder(queries):
             return app.response_class(
-                json_generator(query_func(queries), root='results'),
+                json_generator(
+                    query_func(queries),
+                    root='results', expand=is_expand()
+                ),
                 mimetype='application/json'
             )
-        responder.__name__ = b'query_%s' % name.encode('utf8')
+        responder.__name__ = 'query_{0}'.format(name)
         return responder
     return make_responder
 
@@ -117,14 +127,24 @@ def resource_list(name):
     def make_responder(list_all):
         def responder():
             return app.response_class(
-                json_generator(list_all(), root=name),
+                json_generator(list_all(), root=name, expand=is_expand()),
                 mimetype='application/json'
             )
-        responder.__name__ = b'all_%s' % name.encode('utf8')
+        responder.__name__ = 'all_{0}'.format(name)
         return responder
     return make_responder
 
 
+def _get_unique_table_field_values(model, field, sort_field):
+    """ retrieve all unique values belonging to a key from a model """
+    if field not in model.all_keys() or sort_field not in model.all_keys():
+        raise KeyError
+    with g.lib.transaction() as tx:
+        rows = tx.query('SELECT DISTINCT "{0}" FROM "{1}" ORDER BY "{2}"'
+                        .format(field, model._table, sort_field))
+    return [row[0] for row in rows]
+
+
 class IdListConverter(BaseConverter):
     """Converts comma separated lists of ids in urls to integer lists.
     """
@@ -183,8 +203,11 @@ def all_items():
 @app.route('/item/<int:item_id>/file')
 def item_file(item_id):
     item = g.lib.get_item(item_id)
-    response = flask.send_file(item.path, as_attachment=True,
-                               attachment_filename=os.path.basename(item.path))
+    response = flask.send_file(
+        util.py3_path(item.path),
+        as_attachment=True,
+        attachment_filename=os.path.basename(item.path),
+    )
     response.headers['Content-Length'] = os.path.getsize(item.path)
     return response
 
@@ -195,6 +218,17 @@ def item_query(queries):
     return g.lib.items(queries)
 
 
+@app.route('/item/values/<string:key>')
+def item_unique_field_values(key):
+    sort_key = flask.request.args.get('sort_key', key)
+    try:
+        values = _get_unique_table_field_values(beets.library.Item, key,
+                                                sort_key)
+    except KeyError:
+        return flask.abort(404)
+    return flask.jsonify(values=values)
+
+
 # Albums.
 
 @app.route('/album/<idlist:ids>')
@@ -219,7 +253,21 @@ def album_query(queries):
 @app.route('/album/<int:album_id>/art')
 def album_art(album_id):
     album = g.lib.get_album(album_id)
-    return flask.send_file(album.artpath)
+    if album.artpath:
+        return flask.send_file(album.artpath)
+    else:
+        return flask.abort(404)
+
+
+@app.route('/album/values/<string:key>')
+def album_unique_field_values(key):
+    sort_key = flask.request.args.get('sort_key', key)
+    try:
+        values = _get_unique_table_field_values(beets.library.Album, key,
+                                                sort_key)
+    except KeyError:
+        return flask.abort(404)
+    return flask.jsonify(values=values)
 
 
 # Artists.
@@ -264,9 +312,9 @@ class WebPlugin(BeetsPlugin):
         })
 
     def commands(self):
-        cmd = ui.Subcommand('web', help='start a Web interface')
-        cmd.parser.add_option('-d', '--debug', action='store_true',
-                              default=False, help='debug mode')
+        cmd = ui.Subcommand('web', help=u'start a Web interface')
+        cmd.parser.add_option(u'-d', u'--debug', action='store_true',
+                              default=False, help=u'debug mode')
 
         def func(lib, opts, args):
             args = ui.decargs(args)
@@ -276,9 +324,12 @@ class WebPlugin(BeetsPlugin):
                 self.config['port'] = int(args.pop(0))
 
             app.config['lib'] = lib
+            # Normalizes json output
+            app.config['JSONIFY_PRETTYPRINT_REGULAR'] = False
+
             # Enable CORS if required.
             if self.config['cors']:
-                self._log.info('Enabling CORS with origin: {0}',
+                self._log.info(u'Enabling CORS with origin: {0}',
                                self.config['cors'])
                 from flask.ext.cors import CORS
                 app.config['CORS_ALLOW_HEADERS'] = "Content-Type"
@@ -287,7 +338,7 @@ class WebPlugin(BeetsPlugin):
                 }
                 CORS(app)
             # Start the web application.
-            app.run(host=self.config['host'].get(unicode),
+            app.run(host=self.config['host'].as_str(),
                     port=self.config['port'].get(int),
                     debug=opts.debug, threaded=True)
         cmd.func = func
